import { __awaiter } from 'tslib';
import { Injectable, Inject, Optional, NgModule } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { filter, take } from 'rxjs/operators';
import * as IpfsCore from 'ipfs-core';

/**
 * Wrapper service of ipfs-core.
 *
 * @dynamic
 */
class AngularIpfsService {
    constructor(ipfsCore, _window) {
        this.status$ = new BehaviorSubject('INIT');
        this.node$ = new BehaviorSubject(null);
        this._ipfsCore = ipfsCore ? ipfsCore : IpfsCore;
        this._window = _window || window;
    }
    start(options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const status = this.status$.getValue();
            if (status === 'INIT' && this._window.ipfs && this._window.ipfs.enable) {
                console.log('Ng-ipfs: Found window.ipfs');
                const node = yield this._window.ipfs.enable({ commands: ['id'] });
                this.node$.next(node);
                this.status$.next('STARTED');
                return;
            }
            switch (status) {
                case 'STARTED':
                case 'STARTING':
                    console.log('Ng-ipfs: IPFS is already started');
                    break;
                case 'INIT':
                    // eslint-disable-next-line no-console
                    console.time('Ng-ipfs: IPFS is started');
                    this.status$.next('STARTING');
                    try {
                        const node = yield this._ipfsCore.create(options);
                        this.node$.next(node);
                        this.status$.next('STARTED');
                        // eslint-disable-next-line no-console
                        console.timeEnd('Ng-ipfs: IPFS is started');
                    }
                    catch (error) {
                        console.error('Ng-ipfs: IPFS init error:', error);
                        this.node$.next(null);
                        this.status$.next('INIT');
                    }
                    break;
            }
        });
    }
    get() {
        const status = this.status$.getValue();
        if (status === 'INIT') {
            throw new Error('Ng-ipfs: Ipfs node is not started yet. Please call "start()" before.');
        }
        // When status is 'STARTING' or 'STARTED'.
        return this.node$
            .pipe(filter((v) => !!v), take(1))
            .toPromise();
    }
}
AngularIpfsService.decorators = [
    { type: Injectable }
];
AngularIpfsService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: ['IpfsCore',] }, { type: Optional }] },
    { type: undefined, decorators: [{ type: Inject, args: ['Window',] }, { type: Optional }] }
];

class AngularIpfsModule {
}
AngularIpfsModule.decorators = [
    { type: NgModule, args: [{
                providers: [AngularIpfsService],
            },] }
];

/*
 * Public API Surface of angular-ipfs-service
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AngularIpfsModule, AngularIpfsService };
//# sourceMappingURL=angular-ipfs-service.js.map
